@@ -15,7 +15,7 @@ def dumpstacks(signal, frame):
             code.append('File: "%s", line %d, in %s' % (filename, lineno, name))
             if line:
                 code.append("  %s" % (line.strip()))
-    print "\n".join(code)
+    print("\n".join(code))
 
 try:
     import signal
@@ -23,4 +23,3 @@ try:
 except Exception as e:
     # Silently ignore failures installing this handler (probably won't work on Windows)
     pass
-
