@@ -8,7 +8,7 @@ import ast
 import sys, struct, time, os, datetime, platform
 import math, re
 import matplotlib
-if platform.system() != "Darwin":
+if platform.system() != "Darwin" and os.getenv("MPLBACKEND") is None:
     # on MacOS we can't set WxAgg here as it conflicts with the MacOS version
     matplotlib.use('WXAgg')
 from math import *
