@@ -14,7 +14,7 @@
 #
 # Responsible  : Jaime Machuca
 #
-# License      : CC BY-NC-SA
+# License      : GNU GPL version 3
 #
 # Editor Used  : Xcode 6.1.1 (6A2008a)
 #
@@ -31,6 +31,8 @@ from datetime import datetime
 # Module Dependent Headers
 import requests, json, socket, StringIO
 import xml.etree.ElementTree as ET
+import urllib
+from sc_ExifWriter import ExifWriter
 
 # Own Headers
 import ssdp
@@ -75,7 +77,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : None
 #
-#   Autor           : Jaime Machuca, Randy Mackay
+#   Author           : Jaime Machuca, Randy Mackay
 #
 #****************************************************************************
 
@@ -84,17 +86,17 @@ class SmartCamera_SonyQX():
         # record instance
         self.u8Instance = u8Instance
         self.sConfigGroup = "Camera%d" % self.u8Instance
-        
+
         # background image processing variables
         self.u32ImgCounter = 0              # num images requested so far
 
         # latest image captured
         self.sLatestImageURL = None         # String with the URL to the latest image
-        
+
         # latest image downloaded
         self.sLatestImageFilename = None    #String with the Filename for the last downloaded image
         self.sLatestFileName = None         #String with the camera file name for the last image taken
-        
+
         self.vehicleLat = 0.0              # Current Vehicle Latitude
         self.vehicleLon = 0.0              # Current Vehicle Longitude
         self.vehicleHdg = 0.0              # Current Vehicle Heading
@@ -121,7 +123,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : String with object instance name
 #
-#   Autor           : Randy Mackay
+#   Author           : Randy Mackay
 #
 #****************************************************************************
 
@@ -138,7 +140,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : None
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -146,7 +148,7 @@ class SmartCamera_SonyQX():
         print("Setting up Camera Initial Parameters")
         # Check if we need to do 'startRecMode'
         APIList = self.__sSimpleCall("getAvailableApiList")
-            
+
         # For those cameras which need it
         if 'startRecMode' in (APIList['result'])[0]:
             print("Need to send startRecMode, sending and waiting 5 sec...")
@@ -162,7 +164,14 @@ class SmartCamera_SonyQX():
             time.sleep(1)
 
         # Set Postview Size to Orignial size to get real image filename
+        sResponse = self.__sSimpleCall("getSupportedPostviewImageSize")
+        print("%s" % sResponse)
+
         sResponse = self.__sSimpleCall("setPostviewImageSize", adictParams=["Original"])
+        print("%s" % sResponse)
+
+        sResponse = self.__sSimpleCall("getPostviewImageSize")
+        print("%s" % sResponse)
 
         # Set Mode to Shutter Priority if available
         SupportedModes = self.__sSimpleCall("getSupportedExposureMode")
@@ -172,13 +181,16 @@ class SmartCamera_SonyQX():
         #    self.boSetExposureMode("Manual")
         else:
             print("Error no Shutter Priority Mode")
-                
+
         # Set Target Shutter Speed
         self.boSetShutterSpeed(targetShutterSpeed)
-            
+
         # Set Target ISO Value
         self.boSetISO(targetISOValue)
 
+        #Take Confirmation Picture
+        self.boTakePicture()
+
 #****************************************************************************
 #   Method Name     : boSet_GPS
 #
@@ -189,14 +201,14 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : None
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
     def boSet_GPS(self, mGPSMessage):
         if mGPSMessage.get_type() == 'GLOBAL_POSITION_INT':
-            (self.vehicleLat, self.vehicleLon, self.vehicleHdg, self.vehicleAMSL) = (m.lat*1.0e-7, m.lon*1.0e-7, m.hdg*0.01, m.alt*0.001)
-        
+            (self.vehicleLat, self.vehicleLon, self.vehicleHdg, self.vehicleAMSL) = (mGPSMessage.lat*1.0e-7, mGPSMessage.lon*1.0e-7, mGPSMessage.hdg*0.01, mGPSMessage.alt*0.001)
+
 #****************************************************************************
 #   Method Name     : boSet_Attitude
 #
@@ -207,13 +219,13 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : None
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
-    
+
     def boSet_Attitude(self, mAttitudeMessage):
         if mAttitudeMessage.get_type() == 'ATTITUDE':
-            (self.vehicleRoll, self.vehiclePitch) = (math.degrees(m.roll), math.degrees(m.pitch))
+            (self.vehicleRoll, self.vehiclePitch) = (math.degrees(mAttitudeMessage.roll), math.degrees(mAttitudeMessage.pitch))
 
 #****************************************************************************
 #   Method Name     : __geoRef_write
@@ -224,7 +236,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : None
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -237,6 +249,25 @@ class SmartCamera_SonyQX():
         self.geoRef_writer.write('\n')
         self.geoRef_writer.flush()
 
+#****************************************************************************
+#   Method Name     : __boWriteURLToLog
+#
+#   Description     : Writes images URL to file
+#
+#   Parameters      : sImageFileName    File name of image to be entered into the log
+#
+#   Return Value    : None
+#
+#   Author           : Jaime Machuca
+#
+#****************************************************************************
+
+    # Geo reference log for all the GoPro pictures
+    def __boWriteURLToLog(self, sURLName):
+        self.urlLog_writer.write(sURLName)
+        self.urlLog_writer.write('\n')
+        self.urlLog_writer.flush()
+    
 #****************************************************************************
 #   Method Name     : get_real_Yaw
 #
@@ -246,7 +277,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : None
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -254,7 +285,7 @@ class SmartCamera_SonyQX():
         if (yaw < 0):
             return yaw+360
         return yaw
-    
+
 #****************************************************************************
 #   Method Name     : __writeGeoRefToFile
 #
@@ -265,13 +296,13 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : None
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
-    
+
     def __writeGeoRefToFile(self, sImageFileName1):
         self.__geoRef_write(sImageFileName1)
-    
+
 #****************************************************************************
 #   Method Name     : __openGeoTagLogFile
 #
@@ -282,22 +313,39 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : None
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
     def __openGeoTagLogFile(self):
+        
+        # Verify folder exists
+        if not os.path.exists('/sdcard/log'):
+            os.makedirs('/sdcard/log')
+ 
         #Open GeoTag Log File
         i = 0
         while os.path.exists('/sdcard/log/geoRef%s.log' % i):
-            print('checking /sdcard/log/geoRef%s.log' % i)
+            #print('checking /sdcard/log/geoRef%s.log' % i)
             i += 1
         
+        self.sCurrentGeoRefFilename = '/sdcard/log/geoRef%s.log' % i
         self.geoRef_writer = open('/sdcard/log/geoRef%s.log' % i, 'w', 0)
         self.geoRef_writer.write('Filename, Latitude, Longitude, Alt (AMSL), Roll, Pitch, Yaw\n')
-        
+
         print('Opened GeoTag Log File with Filename: geoRef%s.log' % i)
-    
+        
+        #Open URL Log File
+        i = 0
+        while os.path.exists('/sdcard/log/urlLog%s.log' % i):
+            #print('checking /sdcard/log/urlLog%s.log' % i)
+            i += 1
+        
+        self.sCurrentURLLogFilename = '/sdcard/log/urlLog%s.log' % i
+        self.urlLog_writer = open('/sdcard/log/urlLog%s.log' % i, 'w', 0)
+
+        print('Opened URL Log File with Filename: urlLog%s.log' % i)
+
 #****************************************************************************
 #   Method Name     : __sFindInterfaceIPAddress
 #
@@ -307,7 +355,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : String with the IP Address for the requested interface
 #
-#   Autor           : Jaime Machuca,
+#   Author           : Jaime Machuca,
 #
 #****************************************************************************
 
@@ -329,7 +377,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : JSON encoded string with camera response
 #
-#   Autor           : Andrew Tridgell, Jaime Machuca
+#   Author           : Andrew Tridgell, Jaime Machuca
 #
 #****************************************************************************
 
@@ -341,7 +389,7 @@ class SmartCamera_SonyQX():
                                   data=sData,
                                   headers=adictHeaders).json()
         return sResponse
-    
+
 #****************************************************************************
 #   Method Name     : __sSimpleCall
 #
@@ -356,7 +404,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : JSON encoded string with camera response
 #
-#   Autor           : Andrew Tridgell, Jaime Machuca
+#   Author           : Andrew Tridgell, Jaime Machuca
 #
 #****************************************************************************
 
@@ -379,7 +427,7 @@ class SmartCamera_SonyQX():
 #   Return Value    : String containing the URL for sending commands to the
 #                     Camera
 #
-#   Autor           : Andrew Tridgell, Jaime Machuca
+#   Author           : Andrew Tridgell, Jaime Machuca
 #
 #****************************************************************************
 
@@ -391,10 +439,10 @@ class SmartCamera_SonyQX():
         if len(sRet) == 0:
             return None
         sDMS_URL = sRet[0].location
-        
+
         print("Fetching DMS from %s" % sDMS_URL)
         xmlReq = requests.request('GET', sDMS_URL)
-        
+
         xmlTree = ET.ElementTree(file=StringIO.StringIO(xmlReq.content))
         for xmlElem in xmlTree.iter():
             if xmlElem.tag == '{urn:schemas-sony-com:av}X_ScalarWebAPI_ActionList_URL':
@@ -414,7 +462,7 @@ class SmartCamera_SonyQX():
 #   Return Value    : True if camera has been found
 #                     False if no camera has been found
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -422,22 +470,22 @@ class SmartCamera_SonyQX():
         print ("Checking URL at %s" % self.sCameraURL)
         if self.sCameraURL is None:
             return False
-        
+
         return True
 
 #****************************************************************************
 #   Method Name     : boGetLatestImage
 #
-#   Description     : Dowloads the latest image taken by the camera and then
+#   Description     : Downloads the latest image taken by the camera and then
 #                     saves it to a file name composed by the camera instance
 #                     and image number.
 #
 #   Parameters      : none
 #
-#   Return Value    : True if it was succesful
+#   Return Value    : True if it was successful
 #                     False if no image was downloaded
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -450,6 +498,57 @@ class SmartCamera_SonyQX():
             return True
         return False
 
+#****************************************************************************
+#   Method Name     : boGetAllSessionPictures
+#
+#   Description     : Downloads all the images stored in the URL log file to
+#                     the companion computer.
+#
+#   Parameters      : none
+#
+#   Return Value    : True if it was successful
+#                     False if no image was downloaded
+#
+#   Author           : Jaime Machuca
+#
+#****************************************************************************
+
+    def boGetAllSessionPictures(self, sLogFile):
+        
+        print("Picture Download started")
+        file = open(self.sCurrentURLLogFilename, "r")
+        
+
+        for url in file:
+            url = url.rstrip('\n')
+            #now download link
+            start = self.sLatestImageURL.find('DSC')
+            end = self.sLatestImageURL.find('JPG', start) + 3
+            filename = url[start:end]
+            print("Downloading %s" % filename)
+
+            geotagFile = open(self.sCurrentGeoRefFilename, "r")
+            for line in geotagFile:
+                if filename in line:
+                    currFileLatitude = float(line.split(',')[1])
+                    currFileLongitude = float(line.split(',')[2])
+                    currFileAltitude = float(line.split(',')[3])
+                    print ("%s,%f,%f,%f" % (filename, currFileLatitude, currFileLongitude, currFileAltitude))
+            geotagFile.close()
+            
+            try:
+                req = requests.request('GET', url)
+                open('/sdcard/log/%s' % filename, 'w').write(req.content)
+                ExifWriter.write_gps('/sdcard/log/%s' % filename, currFileLatitude, currFileLongitude, currFileAltitude)
+            except Exception as ex:
+                template = "An exception of type {0} occurred. Arguments:\n{1!r}"
+                message = template.format(type(ex).__name__, ex.args)
+                print(message)
+            
+        file.close()
+
+        return False
+    
 #****************************************************************************
 #   Method Name     : sGetLatestImageFilename
 #
@@ -460,13 +559,13 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : String containing the image file name
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
     def sGetLatestImageFilename(self):
         return self.sLatestImageFilename
-    
+
 #****************************************************************************
 #   Method Name     : u32GetImageCounter
 #
@@ -476,7 +575,7 @@ class SmartCamera_SonyQX():
 #
 #   Return Value    : Integer with the number of images
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -490,10 +589,10 @@ class SmartCamera_SonyQX():
 #
 #   Parameters      : None
 #
-#   Return Value    : True if succesful
-#                     False if Error Recieved
+#   Return Value    : True if successful
+#                     False if Error Received
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -501,12 +600,12 @@ class SmartCamera_SonyQX():
 
         # Send command to set Exposure Mode
         sResponse = self.__sSimpleCall("actZoom", adictParams=["in","1shot"])
-        
-        # Check response for a succesful result
+
+        # Check response for a successful result
         if 'result' in sResponse:
             print ("Zoomed in")
             return True
-        
+
         # In case of an error, return false
         print ("Failed to Zoom")
         return False
@@ -518,19 +617,19 @@ class SmartCamera_SonyQX():
 #
 #   Parameters      : None
 #
-#   Return Value    : True if succesful
-#                     False if Error Recieved
+#   Return Value    : True if successful
+#                     False if Error Received
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
     def boZoomOut(self):
-        
+
         # Send command to set Exposure Mode
         sResponse = self.__sSimpleCall("actZoom", adictParams=["out","1shot"])
-        
-        # Check response for a succesful result
+
+        # Check response for a successful result
         if 'result' in sResponse:
             print ("Zoomed out")
             return True
@@ -547,29 +646,29 @@ class SmartCamera_SonyQX():
 #   Parameters      : Exposure Mode String
 #                     Program Auto, Aperture, Shutter, Manual, Intelligent Auto, Superior Auto
 #
-#   Return Value    : True if succesful
-#                     False if Error Recieved
+#   Return Value    : True if successful
+#                     False if Error Received
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
     def boSetExposureMode(self,sExposureMode):
         # Send command to set Exposure Mode
         sResponse = self.__sSimpleCall("setExposureMode", adictParams=[sExposureMode])
-        
-        # Check response for a succesful result
+
+        # Check response for a successful result
         if 'result' in sResponse:
             time.sleep(0.25)
             sResponse = self.__sSimpleCall("getExposureMode")
-            
+
             if sExposureMode not in sResponse["result"]:
                 print ("Failed to set Exposure Mode, current value: %s" %sResponse["result"])
                 return False
-            
+
             print ("Exposure Mode set to %s" % sExposureMode)
             return True
-        
+
         # In case of an error, return false
         print ("Failed to set Exposure Mode")
         return False
@@ -583,10 +682,10 @@ class SmartCamera_SonyQX():
 #                     i.e. 1/1000 = 1000
 #                     NOTE: This will only work for shutter speeds smaller than 1 sec
 #
-#   Return Value    : True if succesful
-#                     False if Error Recieved
+#   Return Value    : True if successful
+#                     False if Error Received
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -596,19 +695,19 @@ class SmartCamera_SonyQX():
 
         # Send command to set Exposure Mode
         sResponse = self.__sSimpleCall("setShutterSpeed", adictParams=[sShutterSpeed])
-            
-        # Check response for a succesful result
+
+        # Check response for a successful result
         if 'result' in sResponse:
             time.sleep(0.25)
             sResponse = self.__sSimpleCall("getShutterSpeed")
-            
+
             if sShutterSpeed not in sResponse["result"]:
                 print ("Failed to set Shutter Speed, current value: %s" %sResponse["result"])
                 return False
 
             print ("Shutter Speed set to %s" % sShutterSpeed)
             return True
-        
+
         # In case of an error, return false
         print ("Failed to set Shutter Speed")
         return False
@@ -621,10 +720,10 @@ class SmartCamera_SonyQX():
 #   Parameters      : F number * 10
 #                     i.e. F 2.8 = 28
 #
-#   Return Value    : True if succesful
-#                     False if Error Recieved
+#   Return Value    : True if successful
+#                     False if Error Received
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -635,19 +734,19 @@ class SmartCamera_SonyQX():
 
         # Send command to set Exposure Mode
         sResponse = self.__sSimpleCall("setFNumber", adictParams=[sFValue])
-        
-        # Check response for a succesful result
+
+        # Check response for a successful result
         if 'result' in sResponse:
             time.sleep(0.25)
             sResponse = self.__sSimpleCall("getFNumber")
-            
+
             if sFValue not in sResponse["result"]:
                 print ("Failed to set aperture, current value: %s" %sResponse["result"])
                 return False
-            
+
             print ("Aperture set to %s" % sFValue)
             return True
-            
+
         # In case of an error, return false
         print ("Failed to set aperture")
         return False
@@ -660,31 +759,31 @@ class SmartCamera_SonyQX():
 #   Parameters      : ISO Value
 #                     80, 100, 1000, 3200, etc...
 #
-#   Return Value    : True if succesful
-#                     False if Error Recieved
+#   Return Value    : True if successful
+#                     False if Error Received
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
     def boSetISO(self,u16ISO):
         # Create ISO String
         sISO = str(u16ISO)
-        
+
         # Send command to set Exposure Mode
         sResponse = self.__sSimpleCall("setIsoSpeedRate", adictParams=[sISO])
-       
-        # Check response for a succesful result
+
+        # Check response for a successful result
         if 'result' in sResponse:
             sResponse = self.__sSimpleCall("getIsoSpeedRate")
 
             if sISO not in sResponse["result"]:
                 print ("Failed to Set ISO, current value: %s" %sResponse["result"])
                 return False
-            
+
             print ("ISO set to %s" % sISO)
             return True
-        
+
         # In case of an error, return false
         print ("Failed to Set ISO")
         return False
@@ -697,10 +796,10 @@ class SmartCamera_SonyQX():
 #
 #   Parameters      : Image file name, position, orientation
 #
-#   Return Value    : True if succesful
-#                     False if no URL was recieved for the image
+#   Return Value    : True if successful
+#                     False if no URL was received for the image
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -715,10 +814,10 @@ class SmartCamera_SonyQX():
 #
 #   Parameters      : none
 #
-#   Return Value    : True if succesful
-#                     False if no URL was recieved for the image
+#   Return Value    : True if successful
+#                     False if no URL was received for the image
 #
-#   Autor           : Jaime Machuca
+#   Author           : Jaime Machuca
 #
 #****************************************************************************
 
@@ -726,7 +825,7 @@ class SmartCamera_SonyQX():
         # Send command to take picture to camera
         sResponse = self.__sSimpleCall("actTakePicture")
 
-        # Check response for a succesful result and save latest image URL
+        # Check response for a successful result and save latest image URL
         if 'result' in sResponse:
             self.sLatestImageURL = sResponse['result'][0][0]
 
@@ -736,6 +835,7 @@ class SmartCamera_SonyQX():
             print("image URL: %s" % self.sLatestImageURL)
             print("image Name: %s" % self.sLatestFileName)
             self.__boAddGeotagToLog(self.sLatestFileName)
+            self.__boWriteURLToLog(self.sLatestImageURL)
 
             self.u32ImgCounter = self.u32ImgCounter+1
             return True
@@ -767,13 +867,13 @@ class SmartCamera_SonyQX():
                 # display image
                 cv2.imshow ('image_display', self.get_latest_image())
             else:
-                print "no image"
-    
+                print("no image")
+
             # check for ESC key being pressed
             k = cv2.waitKey(5) & 0xFF
             if k == 27:
                 break
-    
+
             # take a rest for a bit
             time.sleep(0.01)
 
